// Copyright (c) 1999-2009  Pavel Rychly

#include <finlib/config.hh>
#include <finlib/consumer.hh>
#include <finlib/fromtof.hh>
#include <unistd.h>
#include <iostream>
#include <cstdlib>

using namespace std;

void usage()
{
    cerr << "usage: mkdtext [-b] [-g] [-a] DELTATEXT_PATH [FILENAME]\n"
	 << "    -b    create big_delta_text\n"
	 << "    -g    create giga_delta_text\n"
	 << "    -a    append positions to the end\n"
	;
}

int main (int argc, char **argv) 
{
    const char *progname = argv[0];
    TextConsumer::TextType ttype = TextConsumer::tdelta;
    bool append_to_end = false;
    int seg_size = 128;
    char *outname;
    FILE *input = stdin;
    {
	// process options
	int c;
	while ((c = getopt (argc, argv, "?habg")) != -1)
	    switch (c) {
	    case 'a':
		append_to_end = true;
		break;
	    case 'b':
		ttype = TextConsumer::tdeltabig;
		break;
	    case 'g':
		ttype = TextConsumer::tdeltagiga;
		break;
	    case 'h':
	    case '?':
		usage();
		return 2;
	    default:
		cerr << "unknown option (" << c << ")\n";
		usage();
		return 2;
	    }
	
	if (optind < argc) {
	    outname = argv [optind++];
	    if (optind < argc)
		input = fopen (argv [optind], "r");
	} else {
	    usage();
	    return 2;
	}
    }

    string path = string (outname) + ".text";
    try {
	TextConsumer *dtextcons = 
	    TextConsumer::create (ttype, path);
	FromFile<int> ci (input);
	while (!ci) {
	    dtextcons->put (*ci);
	    ++ci;
	}
	delete dtextcons;
    } catch (exception &e) {
	cerr << e.what() << endl;
	return 1;
    }
    return 0;
}
